/**
 * VUCAKO Bench in Java: main program.
 *
 * @author  $Author: pepca $
 * @version 1.016 $Rev: 94 $ $Date: 2007-03-18 21:21:44 +0100 (ne, 18 III 2007) $
 * @since   0.006
 */
public class Bench
{
  /** Maximum ordinal number of implemented test. */
  protected final static int MAX_TEST_NUMBER = 24;

  /** Default test number. */
  protected final static int DEFAULT_TEST_NUMBER = 1;

  private final static String REV = "$Rev: 94 $";

  /** Actual revision number */
  protected final static int REVISION = Integer.parseInt( REV.substring( 6, REV.length() - 2 ) );

  /**
   * Formats integer in &quot;%.3f&quot; C-format.
   *
   * @param  value Number to be formatted.
   * @return String representation of the given number.
   */
  public static String formatSec ( long value )
  {
    return( Long.toString( value/1000L ) + "." +
            Long.toString( (value/100L) % 10L ) +
            Long.toString( (value/10L) % 10L ) +
            Long.toString( value%10L ) );
  }

  /**
   * Runs the given benchmark.
   *
   * @param  b Initialized <code>Benchamrk</code> instance.
   * @return Runing time in millicesonds.
   */
  protected static long runTest ( Benchmark b )
  {
    long elapsed = b.runTest();
    System.out.println( b.testResult() );
    System.out.print( "Elapsed time: " + formatSec( elapsed ) + " sec " );
    return elapsed;
  }

  /**
   * Main jBench routine.
   *
   * @param  args Arguments passed to the program.
   */
  public static void main ( String[] args )
  {
    System.out.println( "VUCAKO jBench ver 1.016, rev" + REVISION +
                        " (c) 2001-07 Josef.Pelikan@mff.cuni.cz (Open Src)" );

    int testNo  = DEFAULT_TEST_NUMBER;
    boolean alt = false;
    int clock   = 100;                      // default clock spped in MHz
    String cpu  = "unknown";                // default CPU description

    int argIndex;
    for ( argIndex = 0; argIndex < args.length; argIndex++ )
    {
      if ( args[argIndex].equals( "-alt" ) ) // alternative
          alt = true;
      else
      if ( args[argIndex].equals( "-c" ) )  // clock speed, CPU description
      {
        if ( argIndex+1 < args.length )
        {
          clock = Integer.parseInt( args[++argIndex] );
          if ( argIndex+1 < args.length )
            cpu = args[++argIndex];
        }
      }
      else
      if ( args[argIndex].equals( "-t" ) )  // self-test
      {
        RandomJames rnd = new RandomJames();
        System.out.println( "RandomJames pseudo-random generator test: " +
                            (rnd.test() ? "passed" : "failed") );
        return;
      }
      else
      if ( args[argIndex].equals( "-v" ) )  // explicit Java VM version
      {
        if ( argIndex+1 < args.length )
          VMVersion.setVersion( args[++argIndex] );
      }
      else                                  // test #
      {
        testNo = Integer.parseInt( args[argIndex] );
        if ( testNo < 1 ) testNo = 1;
        else
        if ( testNo > MAX_TEST_NUMBER ) testNo = MAX_TEST_NUMBER;
      }
    }
    System.out.println( "Test #" + Integer.toString(testNo) +
                        (alt ? "(alt)" : "") + " on \"" + cpu + "\" (" + clock + "MHz):" );

    long elapsed = 0L;                      // test running time in milliseconds
    switch ( testNo )
    {
      case  1:                              // Eratosthen
        elapsed = runTest( alt ? new BenchmarkEratosthenAlt() : new BenchmarkEratosthen() );
        break;

      case  2:                              // Flawed image transposition (16MB image)
      case  3:                              // Flawed image transposition (64MB image)
        elapsed = runTest( new BenchmarkTranspose( testNo, false, alt ) );
        break;

      case  4:                              // Arbitrage sequence lookup
        elapsed = runTest( new BenchmarkArbitrage() );
        break;

      case  5:                              // Needle-throwing simulation
        elapsed = runTest( new BenchmarkNeedle() );
        break;

      case  6:                              // Fast memory-copy
        elapsed = runTest( new BenchmarkMemoryCopy() );
        break;

      case  7:                              // Monte-Carlo form-factor
        elapsed = runTest( new BenchmarkFormFactor() );
        break;

      case  8:                              // Merge-sort on disk
      case 19:                              // Merge-sort on disk (big version)
        elapsed = runTest( alt ? new BenchmarkMergeSortAlt( testNo ) : new BenchmarkMergeSort( testNo ) );
        break;

      case  9:                              // Wavelet transform in memory
        elapsed = runTest( new BenchmarkWaveletMemory() );
        break;

      case 10:                              // Wavelet transform on disk
      case 20:                              // Wavelet transform on disk (big version)
        elapsed = runTest( new BenchmarkWaveletDisk( testNo ) );
        break;

      case 11:                              // Adaptive K-D tree on disk
      case 21:                              // Adaptive K-D tree on disk (big version)
        elapsed = runTest( new BenchmarkAKDTree( testNo ) );
        break;

      case 12:                              // Quick-sort in virtual memory (64MB array)
      case 13:                              // Quick-sort in virtual memory (256MB array)
      case 14:                              // Quick-sort in virtual memory (1GB array)
        elapsed = runTest( new BenchmarkQuickSort( testNo ) );
        break;

      case 15:                              // Garbage collection in 64MB RAM
        elapsed = runTest( new BenchmarkGC() );
        break;

      case 17:                              // Image transposition (16MB image)
      case 18:                              // Image transposition (64MB image)
        elapsed = runTest( new BenchmarkTranspose( testNo, true, alt ) );
        break;

      case 22:                              // Parallel SHA-1 digest
      case 23:                              // Context-switch efficiency (2 threads)
        elapsed = runTest( new BenchmarkThread( testNo ) );
        break;

      case 24:                              // Parallel sort in virtual memory (1GB array + tmp 1GB)
        elapsed = runTest( new BenchmarkParallelSort( 16 ) );
        break;
    }

      // result processing ["clock","cpu","elapsed"]:
    double enhancement = 2.0;
    if ( elapsed > 0L )
    {
      BenchResults res = new BenchResults( null );
      enhancement = res.addResult( testNo, clock, cpu, elapsed );
      res.close();
      if ( enhancement == 0.0 )
        System.out.println( "(NEW)" );
      else
        System.out.println( "(" + Integer.toString( (int)(enhancement * 100.0) ) +
                            "." + Integer.toString( (int)(enhancement * 1000.0) % 10 ) +
                                  Integer.toString( (int)(enhancement * 10000.0) % 10 ) +
                            "%)" );
    }

    try
    {
      Thread.sleep( (enhancement <= 1.0) ? 20000L : 12000L );
    }
    catch ( InterruptedException e ) {}
  }

}
