/**
 * Abstract benchmark object (one test algorithm).
 *
 * @author  $Author: pepca $
 * @version 1.012 $Rev: 80 $ $Date: 2007-02-23 21:36:26 +0100 (pá, 23 II 2007) $
 * @since   0.007
 */
public abstract class Benchmark
{
  /**
   * Test (benchmark) ordinal number.
   *
   * @return Ordinal number of the test.
   */
  public int testNumber ()
  {
    return -1;
  }

  /**
   * The benchmark algorithm itself. Needs not be re-entrant but can be iterated.
   */
  abstract protected void test ();

  /**
   * Runs the test algorithm.
   *
   * @return Number of milliseconds.
   */
  public long runTest ()
  {
    long startTime = System.currentTimeMillis();
    test();
    long stopTime = System.currentTimeMillis();
    return( stopTime - startTime );
  }

  /**
   * Result string of the test. Should be called after the <code>runTest()</code>
   * was finished.
   *
   * @return Result of the test.
   */
  public String testResult ()
  {
    return null;
  }

}
