/*
  sha1.h

  SHA-1 digest in C by Steve Reid <steve@edmweb.com> (100% Public Domain).
  Adopted by Josef Pelikan.

  $Rev: 93 $
  $Date: 2007-03-17 16:25:56 +0100 (so, 17 III 2007) $

  Test Vectors (from FIPS PUB 180-1):

  "abc"
  A9993E36 4706816A BA3E2571 7850C26C 9CD0D89D

  "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
  84983E44 1C3BD26E BAAE4AA1 F95129E5 E54670F1

  A million repetitions of "a"
  34AA973C D4C4DAA4 F61EEB2B DBAD2731 6534016F
*/

#ifndef _SHA1_H
#define _SHA1_H

class Sha1
{
  //-----------------------------------------------------------
  //  State variables, support:

protected:

  unsigned32 state[ 5 ];

  unsigned32 count[ 2 ];

  union
  {
    byte       c[ 64 ];
    unsigned32 l[ 16 ];
  };

  /** Transforms 512 bits of data. */
  void transform ();

  //-----------------------------------------------------------
  //  Public access:

public:

  /** New digest instance is ready for computation. */
  Sha1 ();

  /** [Re-]initialization. */
  void init ();

  /** Processes the given data array. */
  void update ( const byte *data, unsigned len );

  /** Finishes digest computation, returns 160-bit digest. */
  void finish ( byte *digest );

  /** Self-test. Returns true if succeeded. */
  bool test ();
};

#endif
