/*
    test.cc: VUCAKO Bench core (test routines).

    $Rev: 90 $
    $Date: 2007-03-13 10:46:11 +0100 (út, 13 III 2007) $

    Copyright (C) 2001-2007 by Josef Pelikan, MFF UK Prague
        http://cgg.ms.mff.cuni.cz/~pepca/

    VUCAKO Bench is Open Source software (see http://www.opensource.org/
    for description what Open Source is); you may distributed it and/or
    modify it under the terms of GNU General Public License as published
    by the Free Software Foundation; either version 2, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    A copy of the GNU General Public License is available as
    `/usr/doc/copyright/GPL' in the Debian GNU/Linux distribution or on
    the World Wide Web at http://www.gnu.org/copyleft/gpl.html.  You can
    also obtain it by writing to the Free Software Foundation, Inc., 59
    Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "pch.h"

//-----------------------------------------------------------
//  Quick-sort of double[] array:

#define QSMin           12

void qs ( double *a, int len )
{                                           // 64-bit clean
  int i, j;
  if ( len < QSMin )                        // short array => insert-sort
  {
    while ( len > 1 )                       // find minimum and swap it to a[0]
    {
      double min = a[0];
      j = 0;
      for ( i = 1; i < len; i++ )
        if ( a[i] < min )
        {
          min = a[i];
          j = i;
        }
      if ( j != 0 )
      {
        a[j] = a[0];
        a[0] = min;
      }
      a++;
      len--;
    }
    return;
  }

    // the array is long enough for quick-sort:
  double pivot = (a[0] + a[len-1]) * 0.5;
  i = -1; j = len;
  do
  {
    do i++;
    while ( i < len && a[i] <= pivot );

    do j--;
    while ( j >= 0  && a[j] >= pivot );

    if ( j < i ) break;

    double tmp = a[i];
    a[i] = a[j];
    a[j] = tmp;
  }
  while ( true );

    // j < i => sort [0 to j] and [i to len-1]:
  if ( j + 1 < len - i )                    // [0 to j] first
  {
    if ( j > 0 ) qs( a, j+1 );
    qs( a+i, len-i );
  }
  else                                      // [i to len-1] first
  {
    if ( len-i > 1 ) qs( a+i, len-i );
    qs( a, j+1 );
  }
}

//-----------------------------------------------------------
//  Eratosthen (I know this implementation is silly but I'm
//    goint to keep it for result-compatibility reasons):

int eratosthen ( int primeOrder, int primeAssert )
{                                           // 64-bit clean
    startTimer();
    int elapsed;
    int repeat = 0;
    int prime;
    do {
        repeat++;
        CBitMask sieve;
        sieve.assert(2);
        sieve.assert(primeAssert);
        int n = 0;
        prime = 1;
        do {                                // find the next prime
            do prime++;
            while ( sieve.getUnsafe(prime) );
            int k = prime;
            do {
                sieve.setUnsafe(k,true);
                k += prime;
                } while ( k <= primeAssert );
            if ( !(n & 0x7FFF) ) LiveCycle(n,primeOrder);
            } while ( ++n < primeOrder );
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("The %d-th prime number is: %d, elapsed time: %.2f sec\n",
               primeOrder,prime,elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  BYTE matrix:

byte *newImage ( int width, int height )
{
    width  &= -8;
    height &= -8;
    return (byte*)VIRTUAL_ALLOC(width*height);
}

void freeImage ( byte *im, int width, int height )
{
    width  &= -8;
    height &= -8;
    VIRTUAL_FREE(im,width*height);
}

void transpose ( byte *dest, int width, int height, byte *src )
{
    width  &= -4;
    height &= -4;
    unsigned32 *dp1 = (unsigned32*)dest;    // destination pointers
    dest += width;
    unsigned32 *dp2 = (unsigned32*)dest;
    dest += width;
    unsigned32 *dp3 = (unsigned32*)dest;
    dest += width;
    unsigned32 *dp4 = (unsigned32*)dest;
    dest += width;
    unsigned32 *sp0 = (unsigned32*)src;
    unsigned32 *sp;
    unsigned32 dact1, dact2, dact3, dact4, sact;
    int i, j;
    for ( i = 0; i < height; i += 4 ) {     // 4 destination rows
        sp = sp0;
        for ( j = 0; j < width; j += 4 ) {  // 4 destination columns
            sact   = *sp;
            dact1  =    sact        & 0xFF;
            dact2  =   (sact >>  8) & 0xFF;
            dact3  =   (sact >> 16) & 0xFF;
            dact4  =   (sact >> 24) & 0xFF;
            sp += (height >> 2);
            dact1 |= (( sact        & 0xFF) <<  8);
            dact2 |= (((sact >>  8) & 0xFF) <<  8);
            dact3 |= (((sact >> 16) & 0xFF) <<  8);
            dact4 |= (((sact >> 24) & 0xFF) <<  8);
            sp += (height >> 2);
            dact1 |= (( sact        & 0xFF) << 16);
            dact2 |= (((sact >>  8) & 0xFF) << 16);
            dact3 |= (((sact >> 16) & 0xFF) << 16);
            dact4 |= (((sact >> 24) & 0xFF) << 16);
            sp += (height >> 2);
            dact1 |= (( sact        & 0xFF) << 24);
            dact2 |= (((sact >>  8) & 0xFF) << 24);
            dact3 |= (((sact >> 16) & 0xFF) << 24);
            dact4 |= (((sact >> 24) & 0xFF) << 24);
            sp += (height >> 2);
            *dp1++ = dact1;
            *dp2++ = dact2;
            *dp3++ = dact3;
            *dp4++ = dact4;
            }
        sp0++;
        dp1 = (unsigned32*)dest;
        dest += width;
        dp2 = (unsigned32*)dest;
        dest += width;
        dp3 = (unsigned32*)dest;
        dest += width;
        dp4 = (unsigned32*)dest;
        dest += width;
        }
}

#ifdef USE_64_BITS

void transposeCorrect ( byte *dest, int width, int height, byte *src )
{
    width  &= -4;
    height &= -4;
    unsigned64 *dp1 = (unsigned64*)dest;    // destination pointers
    dest += width;
    unsigned64 *dp2 = (unsigned64*)dest;
    dest += width;
    unsigned64 *dp3 = (unsigned64*)dest;
    dest += width;
    unsigned64 *dp4 = (unsigned64*)dest;
    dest += width;
    unsigned64 *dp5 = (unsigned64*)dest;
    dest += width;
    unsigned64 *dp6 = (unsigned64*)dest;
    dest += width;
    unsigned64 *dp7 = (unsigned64*)dest;
    dest += width;
    unsigned64 *dp8 = (unsigned64*)dest;
    dest += width;
    unsigned64 *sp0 = (unsigned64*)src;
    unsigned64 *sp;
    unsigned64 dact1, dact2, dact3, dact4, dact5, dact6, dact7, dact8, sact;
    int i, j;
    for ( i = 0; i < height; i += 8 ) {     // 8 destination rows
        sp = sp0;
        for ( j = 0; j < width; j += 8 ) {  // 8 destination columns
            sact   = *sp;
            dact1  =    sact        & 0xFF;
            dact2  =   (sact >>  8) & 0xFF;
            dact3  =   (sact >> 16) & 0xFF;
            dact4  =   (sact >> 24) & 0xFF;
            dact5  =   (sact >> 32) & 0xFF;
            dact6  =   (sact >> 40) & 0xFF;
            dact7  =   (sact >> 48) & 0xFF;
            dact8  =   (sact >> 56) & 0xFF;
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) <<  8);
            dact2 |= (((sact >>  8) & 0xFF) <<  8);
            dact3 |= (((sact >> 16) & 0xFF) <<  8);
            dact4 |= (((sact >> 24) & 0xFF) <<  8);
            dact5 |= (((sact >> 32) & 0xFF) <<  8);
            dact6 |= (((sact >> 40) & 0xFF) <<  8);
            dact7 |= (((sact >> 48) & 0xFF) <<  8);
            dact8 |= (((sact >> 56) & 0xFF) <<  8);
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 16);
            dact2 |= (((sact >>  8) & 0xFF) << 16);
            dact3 |= (((sact >> 16) & 0xFF) << 16);
            dact4 |= (((sact >> 24) & 0xFF) << 16);
            dact5 |= (((sact >> 32) & 0xFF) << 16);
            dact6 |= (((sact >> 40) & 0xFF) << 16);
            dact7 |= (((sact >> 48) & 0xFF) << 16);
            dact8 |= (((sact >> 56) & 0xFF) << 16);
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 24);
            dact2 |= (((sact >>  8) & 0xFF) << 24);
            dact3 |= (((sact >> 16) & 0xFF) << 24);
            dact4 |= (((sact >> 24) & 0xFF) << 24);
            dact5 |= (((sact >> 32) & 0xFF) << 24);
            dact6 |= (((sact >> 40) & 0xFF) << 24);
            dact7 |= (((sact >> 48) & 0xFF) << 24);
            dact8 |= (((sact >> 56) & 0xFF) << 24);
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 32);
            dact2 |= (((sact >>  8) & 0xFF) << 32);
            dact3 |= (((sact >> 16) & 0xFF) << 32);
            dact4 |= (((sact >> 24) & 0xFF) << 32);
            dact5 |= (((sact >> 32) & 0xFF) << 32);
            dact6 |= (((sact >> 40) & 0xFF) << 32);
            dact7 |= (((sact >> 48) & 0xFF) << 32);
            dact8 |= (((sact >> 56) & 0xFF) << 32);
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 40);
            dact2 |= (((sact >>  8) & 0xFF) << 40);
            dact3 |= (((sact >> 16) & 0xFF) << 40);
            dact4 |= (((sact >> 24) & 0xFF) << 40);
            dact5 |= (((sact >> 32) & 0xFF) << 40);
            dact6 |= (((sact >> 40) & 0xFF) << 40);
            dact7 |= (((sact >> 48) & 0xFF) << 40);
            dact8 |= (((sact >> 56) & 0xFF) << 40);
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 48);
            dact2 |= (((sact >>  8) & 0xFF) << 48);
            dact3 |= (((sact >> 16) & 0xFF) << 48);
            dact4 |= (((sact >> 24) & 0xFF) << 48);
            dact5 |= (((sact >> 32) & 0xFF) << 48);
            dact6 |= (((sact >> 40) & 0xFF) << 48);
            dact7 |= (((sact >> 48) & 0xFF) << 48);
            dact8 |= (((sact >> 56) & 0xFF) << 48);
            sp += (height >> 3);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 56);
            dact2 |= (((sact >>  8) & 0xFF) << 56);
            dact3 |= (((sact >> 16) & 0xFF) << 56);
            dact4 |= (((sact >> 24) & 0xFF) << 56);
            dact5 |= (((sact >> 32) & 0xFF) << 56);
            dact6 |= (((sact >> 40) & 0xFF) << 56);
            dact7 |= (((sact >> 48) & 0xFF) << 56);
            dact8 |= (((sact >> 56) & 0xFF) << 56);
            sp += (height >> 3);
            *dp1++ = dact1;
            *dp2++ = dact2;
            *dp3++ = dact3;
            *dp4++ = dact4;
            *dp5++ = dact5;
            *dp6++ = dact6;
            *dp7++ = dact7;
            *dp8++ = dact8;
            }
        sp0++;
        dp1 = (unsigned64*)dest;
        dest += width;
        dp2 = (unsigned64*)dest;
        dest += width;
        dp3 = (unsigned64*)dest;
        dest += width;
        dp4 = (unsigned64*)dest;
        dest += width;
        dp5 = (unsigned64*)dest;
        dest += width;
        dp6 = (unsigned64*)dest;
        dest += width;
        dp7 = (unsigned64*)dest;
        dest += width;
        dp8 = (unsigned64*)dest;
        dest += width;
        }
}

#else     // USE_64_BITS

void transposeCorrect ( byte *dest, int width, int height, byte *src )
{
    width  &= -4;
    height &= -4;
    unsigned32 *dp1 = (unsigned32*)dest;    // destination pointers
    dest += width;
    unsigned32 *dp2 = (unsigned32*)dest;
    dest += width;
    unsigned32 *dp3 = (unsigned32*)dest;
    dest += width;
    unsigned32 *dp4 = (unsigned32*)dest;
    dest += width;
    unsigned32 *sp0 = (unsigned32*)src;
    unsigned32 *sp;
    unsigned32 dact1, dact2, dact3, dact4, sact;
    int i, j;
    for ( i = 0; i < height; i += 4 ) {     // 4 destination rows
        sp = sp0;
        for ( j = 0; j < width; j += 4 ) {  // 4 destination columns
            sact   = *sp;
            dact1  =    sact        & 0xFF;
            dact2  =   (sact >>  8) & 0xFF;
            dact3  =   (sact >> 16) & 0xFF;
            dact4  =   (sact >> 24) & 0xFF;
            sp += (height >> 2);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) <<  8);
            dact2 |= (((sact >>  8) & 0xFF) <<  8);
            dact3 |= (((sact >> 16) & 0xFF) <<  8);
            dact4 |= (((sact >> 24) & 0xFF) <<  8);
            sp += (height >> 2);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 16);
            dact2 |= (((sact >>  8) & 0xFF) << 16);
            dact3 |= (((sact >> 16) & 0xFF) << 16);
            dact4 |= (((sact >> 24) & 0xFF) << 16);
            sp += (height >> 2);
            sact   = *sp;
            dact1 |= (( sact        & 0xFF) << 24);
            dact2 |= (((sact >>  8) & 0xFF) << 24);
            dact3 |= (((sact >> 16) & 0xFF) << 24);
            dact4 |= (((sact >> 24) & 0xFF) << 24);
            sp += (height >> 2);
            *dp1++ = dact1;
            *dp2++ = dact2;
            *dp3++ = dact3;
            *dp4++ = dact4;
            }
        sp0++;
        dp1 = (unsigned32*)dest;
        dest += width;
        dp2 = (unsigned32*)dest;
        dest += width;
        dp3 = (unsigned32*)dest;
        dest += width;
        dp4 = (unsigned32*)dest;
        dest += width;
        }
}

#endif    // USE_64_BITS

int imageTransposition ( int width, int height, int iter )
{
    startTimer();
    int elapsed;
    int repeat = 0;
    do {
        repeat++;
        byte *im1 = newImage(width,height);
        if ( !im1 ) return 0;
        byte *im2 = newImage(height,width);
        if ( !im2 ) {
            freeImage(im1,width,height);
            return 0;
            }
        int i;
        for ( i = 0; i++ < iter; ) {
            transpose(im2,height,width,im1);
            transpose(im1,width,height,im2);
            LiveCycle(i,iter);
            }
        freeImage(im2,height,width);
        freeImage(im1,width,height);
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Transpose the [%d,%d] image, elapsed time: %.2f sec\n",
               width,height,elapsed*0.001);
    return elapsed;
}

int imageTranspositionCorrect ( int width, int height, int iter )
{
    startTimer();
    int elapsed;
    int repeat = 0;
    do {
        repeat++;
        byte *im1 = newImage(width,height);
        if ( !im1 ) return 0;
        byte *im2 = newImage(height,width);
        if ( !im2 ) {
            freeImage(im1,width,height);
            return 0;
            }
        int i;
        for ( i = 0; i++ < iter; ) {
            transposeCorrect(im2,height,width,im1);
            transposeCorrect(im1,width,height,im2);
            LiveCycle(i,iter);
            }
        freeImage(im2,height,width);
        freeImage(im1,width,height);
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Transpose the [%d,%d] image, elapsed time: %.2f sec\n",
               width,height,elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  arbitrage sequence lookup:

double m[MaxN][MaxN];
int n;

void initInstance ()
{
    n = ArbitrageN;
    int i;
    RandomJames rnd;
    for ( i = 0; i < n; i++ )
        rnd.normalNumbers(1.0,0.05,5,m[i],n);
}

int result[MaxN];
int used[MaxN];
int len;

double best;
int bestSeq[MaxN];
int bestLen;

void solve ( double mul, int i )
{
    int j;
    double act;
    if ( i + 1 == len )
        for ( j = 0; j < n; j++ ) {
            if ( !used[j] )
                if ( (act = mul * m[result[i-1]][j] * m[j][result[0]]) > best ) {
                    best = act;
                    bestLen = len;
                    memcpy(bestSeq,result,i*sizeof(int));
                    bestSeq[i] = j;
                    }
            }
    else
        for ( j = 0; j < n; j++ )
            if ( !used[j] ) {
                used[j] = 1;
                result[i] = j;
                solve(mul*m[result[i-1]][j],i+1);
                used[j] = 0;
                }
}

void solveInstance ()
{
    int i;
    memset(used,0,sizeof(used));
    best = MinBound;
    bestLen = 0;
    for ( len = 2; len < n; len++ ) {
        for ( i = 0; i < n; i++ ) {
            used[i] = 1;
            result[0] = i;
            solve(1.0,1);
            used[i] = 0;
            }
        LiveCycle(len-1,n-1);
        }
    if ( !superBrief )
        if ( bestLen > 0 ) {
            printf("Arbitrage profit:%7.3f, seq:",best);
            for ( i = 0; i < bestLen; i++ )
                printf(" %d",bestSeq[i]+1);
            printf(" %d\n",bestSeq[0]+1);
            }
        else
            puts("No arbitrage sequence exists");
}

//-----------------------------------------------------------
//  needle throwing simulation:

int needleSimulation ()
{
    startTimer();
    unsigned hit, i;
    double y, height, angle;
    int elapsed;
    int repeat = 0;
    do {
        repeat++;
        RandomJames rnd;
        hit = 0;
        for ( i = 0; i++ < NeedleMax; ) {
            y = rnd.uniformNumber() * 2.0;
            angle = rnd.uniformNumber() * 3.141592653589793;
            height = sin(angle);
            if ( y + height >= 2.0 || y - height < 0.0 ) hit++;
            if ( !(i & 0x1FFFFF) ) LiveCycle(i,NeedleMax);
            }
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Needle simulation: N = %dM, elapsed time: %.2f sec, ratio = %.9f\n",
               NeedleMax/1000000,elapsed*0.001,(hit+0.0)/NeedleMax);
    return elapsed;
}

//-----------------------------------------------------------
//  fast mem-copy:

int memCopy ()
{
    startTimer();
    int elapsed;
    int repeat = 0;
    do {
        repeat++;
        char *a = (char*)VIRTUAL_ALLOC(MemCopyMax);
        if ( !a ) return 0;
        char *b = (char*)VIRTUAL_ALLOC(MemCopyMax);
        if ( !b ) {
            VIRTUAL_FREE(a,MemCopyMax);
            return 0;
            }
        int i;
        for ( i = 0; i++ < MemCopyIter; ) {
            memcpy(b,a,MemCopyMax);
            memcpy(a,b,MemCopyMax);
            if ( i & 1 ) LiveCycle(i,MemCopyIter);
            }
        VIRTUAL_FREE(a,MemCopyMax);
        VIRTUAL_FREE(b,MemCopyMax);
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Fast mem-copy: array size = %d MB, iterations = %d, elapsed time: %.2f sec\n",
               MemCopyMax>>20,MemCopyIter,elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  Monte-Carlo form-factor between two identical rectangles:

int formFactor ()
{
    startTimer();
    int i;
    unsigned fullHits, halfCount, halfHits;
    double x0, y0;                          // ray origin
    double x1, y1, z1;                      // ray direction
    double x2, y2;
    double tmp;
    bool half;
    int elapsed;
    int repeat = 0;
    do {
        repeat++;
        fullHits = halfCount = halfHits = 0;
        RandomJames rnd;
        for ( i = 0; i++ < FFIter; ) {
            x0 = rnd.uniformNumber();
            y0 = rnd.uniformNumber();
            if ( (half = (x0 < 0.5 && y0 < 0.5)) ) halfCount++;
            do {
                x1 = rnd.uniformNumber() * 2.0 - 1.0;
                y1 = rnd.uniformNumber() * 2.0 - 1.0;
                tmp = x1 * x1 + y1 * y1;
                } while ( tmp >= 1.0 );
            z1 = sqrt( 1.0 - tmp );
            if ( z1 > 0.01 ) {              // there is a chance
                tmp = 1.0 / z1;
                x2 = x0 + tmp * x1;
                y2 = y0 + tmp * y1;
                if ( x2 >= 0.0 && y2 >= 0.0 )
                    if ( x2 < 1.0 && y2 < 1.0 ) {
                        fullHits++;
                        if ( half && x2 < 0.5 && y2 < 0.5 ) halfHits++;
                        }
                }
            if ( !(i & 0xFFFFF) ) LiveCycle(i,FFIter);
            }
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Form-factor: iter = %dM, time: %.2f sec, Ffull =%10.7f, Fhalf =%10.7f\n",
               FFIter/1000000,elapsed*0.001,(fullHits+0.0)/FFIter,(halfHits+0.0)/halfCount);
    return elapsed;
}

//-----------------------------------------------------------
//  natural merge-sort of huge disk files:

#define TestFileName    "sort%d.bin"

bool checkFile ( FileOfDouble &f, int &size )
{
    double prev, act;
    f.seek(0);
    act = f.get();
    size = 1;
    while ( true ) {
        prev = act;
        act = f.get();
        if ( act < 0.0 ) return true;
        size++;
        if ( act < prev ) return false;
        }
}

void initialSplit ( FileOfDouble &from, FileOfDouble &to1, int &seg1, FileOfDouble &to2, int &seg2 )
{
    to1.rewrite();
    seg1 = 0;
    to2.rewrite();
    seg2 = 0;
    from.seek(0);
    double prev, act;
    act = from.get();
    if ( act < 0.0 ) return;
    bool one = true;
    to1.put(act);
    seg1 = 1;

    do {
        prev = act;
        act = from.get();
        if ( act < 0.0 ) return;
        if ( act < prev ) {
            one = !one;
            if ( one ) seg1++;
            else       seg2++;
            }
        if ( one ) to1.put(act);
        else       to2.put(act);
        } while ( true );
}

void mergeAndSplit ( FileOfDouble &from1, FileOfDouble &from2,
                     FileOfDouble &to1, int &seg1, FileOfDouble &to2, int &seg2 )
{
    to1.rewrite();
    seg1 = 0;
    to2.rewrite();
    seg2 = 0;
    from1.seek(0);
    from2.seek(0);
    double prev1, act1;
    double prev2, act2;
    prev1 = from1.get();
    act1 = from1.get();
    prev2 = from2.get();
    act2 = from2.get();
    bool one = true;
    seg1 = 1;

    do {
            // min { prev1, prev2 } should be written out
        if ( prev1 < prev2 ) {
            if ( one ) to1.put(prev1);
            else       to2.put(prev1);
            if ( prev1 > act1 ) {           // copy the rest of from2 segment
                // prev1 > act1, prev1 was processed
                do {
                    if ( one ) to1.put(prev2);
                    else       to2.put(prev2);
                    if ( act2 < prev2 ) break;
                    prev2 = act2;
                    act2 = from2.get();
                    } while ( true );
                // prev2 > act2, prev2 was processed
                if ( act1 < 0.0 || act2 < 0.0 ) // from1 or from2 were exhausted
                    break;
                one = !one;
                if ( one ) seg1++;
                else       seg2++;
                prev2 = act2;
                act2 = from2.get();
                }
            prev1 = act1;
            act1 = from1.get();
            }
        else {                              // prev2 < prev1
            if ( one ) to1.put(prev2);
            else       to2.put(prev2);
            if ( prev2 > act2 ) {           // copy the rest of from1 segment
                // prev2 > act2, prev2 was processed
                do {
                    if ( one ) to1.put(prev1);
                    else       to2.put(prev1);
                    if ( act1 < prev1 ) break;
                    prev1 = act1;
                    act1 = from1.get();
                    } while ( true );
                // prev1 > act1, prev1 was processed
                if ( act1 < 0.0 || act2 < 0.0 ) // from1 or from2 were exhausted
                    break;
                one = !one;
                if ( one ) seg1++;
                else       seg2++;
                prev1 = act1;
                act1 = from1.get();
                }
            prev2 = act2;
            act2 = from2.get();
            }
        } while ( true );

    while ( act1 >= 0.0 ) {                 // from1 wasn't exhausted
        one = seg1 < seg2;
        if ( one ) {                        // put the 1st item
            seg1++;
            to1.put(act1);
            }
        else {
            seg2++;
            to2.put(act1);
            }
        do {
            prev1 = act1;
            act1 = from1.get();
            if ( prev1 > act1 ) break;
            if ( one ) to1.put(act1);
            else       to2.put(act1);
            } while ( true );
        }

    if ( act2 >= 0.0 ) {                    // from2 wasn't exhausted
        one = seg1 < seg2;
        if ( one ) {                        // put the 1st item
            seg1++;
            to1.put(act2);
            }
        else {
            seg2++;
            to2.put(act2);
            }
        do {
            prev2 = act2;
            act2 = from2.get();
            if ( prev2 > act2 ) break;
            if ( one ) to1.put(act2);
            else       to2.put(act2);
            } while ( true );
        }
}

int diskSort ( int size )
  // size == 0 .. normal size, == 1 .. big
{
    printf("init: random generator, bucket-sort\r");
    fflush(stdout);
    char fn1[32], fn2[32], fn3[32], fn4[32];
    sprintf(fn1,TestFileName,1);
    sprintf(fn2,TestFileName,2);
    sprintf(fn3,TestFileName,3);
    sprintf(fn4,TestFileName,4);
    remove(fn1);
    remove(fn2);
    remove(fn3);
    remove(fn4);
    startTimer();
    RandomJames rnd;
    FileOfDouble f1(fn1);
    FileOfDouble f2(fn2);
    FileOfDouble f3(fn3);
    FileOfDouble f4(fn4);
    int seg1, seg2, seg3, seg4;
    seg3 = seg4 = 0;
    int i, j, sortSize;
        // file init:
    if ( size == 0 )
    {
      sortSize = SortSize;
      for ( i = 0; i++ < SortSize/BucketSize; ) {
          double bucket[BucketSize];
          for ( j = 0; j < BucketSize; j++ )
              bucket[j] = rnd.uniformNumber();
          qs(bucket,BucketSize);
          for ( j = 0; j < BucketSize; j++ )
              f3.put(bucket[j]);
          }
    }
    else
    {
      sortSize = SortSizeBig;
      for ( i = 0; i++ < SortSizeBig/BucketSizeBig; ) {
          double bucket[BucketSizeBig];
          for ( j = 0; j < BucketSizeBig; j++ )
              bucket[j] = rnd.uniformNumber();
          qs(bucket,BucketSizeBig);
          for ( j = 0; j < BucketSizeBig; j++ )
              f3.put(bucket[j]);
          }
    }
        // merge-sort:
    initialSplit(f3,f1,seg1,f2,seg2);
    bool ok;
    do {
        printf("%17d %17d\r",seg1,seg2);
        fflush(stdout);
        if ( seg2 == 0 ) {
            ok = checkFile(f1,i);
            break;
            }
        if ( seg1 == 0 ) {
            ok = checkFile(f2,i);
            break;
            }
        mergeAndSplit(f1,f2,f3,seg3,f4,seg4);
        printf("%17d %17d\r",seg3,seg4);
        fflush(stdout);
        if ( seg4 == 0 ) {
            ok = checkFile(f3,i);
            break;
            }
        if ( seg3 == 0 ) {
            ok = checkFile(f4,i);
            break;
            }
        mergeAndSplit(f3,f4,f1,seg1,f2,seg2);
        } while ( true );
        // the right order was checked
    f1.close();
    f2.close();
    f3.close();
    f4.close();
    remove(fn1);
    remove(fn2);
    remove(fn3);
    remove(fn4);
    int elapsed = elapsedTime();
    if ( !superBrief )
        printf("Merge-sort on disk: %s, file size: %d MB, elapsed time: %.2f sec\n",
               (ok && i == sortSize) ? "OK":"fail",(sortSize*sizeof(double))>>20,
               elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  wavelet transform (memory cache test):

void liftMemory ( int n0, int32 *s0, int32 *s1, int32 *d1 )
    // forward TS-transform
{
    int n1 = n0 >> 1;
    int32 *ps0, *ps1, *pd1;
    int i;
        // split:
    ps0 = s0;
    ps1 = s1; pd1 = d1;
    for ( i = 0; i++ < n1; ) {
        *ps1++ = *ps0++;
        *pd1++ = *ps0++;
        }
        // S-transform:
    ps1 = s1; pd1 = d1;
    for ( i = 0; i++ < n1; ) {
        *pd1 -= *ps1;
        *ps1++ += (*pd1++ >> 1);
        }
        // predict:
    ps1 = s1; pd1 = d1;
    ps1[n1] = ps1[n1-1];                    // boundary
    ps1[-1] = ps1[0];
    for ( i = 0; i++ < n1; ) {
        *pd1++ += ((ps1[-1] - ps1[1] + 2) >> 2);
        ps1++;
        }
}

void unliftMemory ( int n0, int32 *s0, int32 *s1, int32 *d1 )
    // inverse TS-transform
{
    int n1 = n0 >> 1;
    int32 *ps0, *ps1, *pd1;
    int i;
        // predict*:
    ps1 = s1; pd1 = d1;
    ps1[n1] = ps1[n1-1];                    // boundary
    ps1[-1] = ps1[0];
    for ( i = 0; i++ < n1; ) {
        *pd1++ -= ((ps1[-1] - ps1[1] + 2) >> 2);
        ps1++;
        }
        // S-transform*:
    ps1 = s1; pd1 = d1;
    for ( i = 0; i++ < n1; ) {
        *ps1 -= (*pd1 >> 1);
        *pd1++ += *ps1++;
        }
        // merge:
    ps0 = s0;
    ps1 = s1; pd1 = d1;
    for ( i = 0; i++ < n1; ) {
        *ps0++ = *ps1++;
        *ps0++ = *pd1++;
        }
}

int waveletMemory ()
{
    startTimer();
    int32 *a = (int32*)VIRTUAL_ALLOC((WaveMaxSize+2)*sizeof(int32));
    if ( !a ) return 0;
    int total = WaveMaxSize;
    int iter = WaveIter;
    int i;
    int el = 0, elOld;
    do {
        int32 *s1 = a + (total >> 1) + 1;
        int32 *d1 = s1 + (total >> 2) + 1;
        for ( i = 0; i++ < iter; ) {
            liftMemory(total>>1,a,s1,d1);
            unliftMemory(total>>1,a,s1,d1);
            }
        elOld = el;
        el = elapsedTime();
        printf("%6.2f",(el-elOld)*0.001);
        fflush(stdout);
        iter += iter;
        } while ( (total /= 2) >= WaveMinSize );
    VIRTUAL_FREE(a,(WaveMaxSize+2)*sizeof(int32));
    int elapsed = elapsedTime();
    if ( superBrief )
        putchar('\r');
    else
        printf("\nWavelet T-S transform: arr: %dMB-%dKB, iter: %d-%d, elapsed time: %.2f sec\n",
               (WaveMaxSize*sizeof(int32))>>20,(WaveMinSize*sizeof(int32))>>10,WaveIter,iter>>1,elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  Wavelet transform (disk cache test):

#define S0FileName      "s0.bin"
#define S1FileName      "s1.bin"
#define D1FileName      "d1.bin"

void liftDisk ( int n0, FileOfDouble &s0, FileOfDouble &s1, FileOfDouble &d1 )
    // forward TS-transform
{
    int n1 = n0 >> 1;
    int i;
        // split + S-transform + prediction:
    s0.seek(0);
    s1.seek(0);
    d1.seek(0);
    double s[3], d[3];
    s[1] = s0.get();
    d[1] = s0.get();
    d[1] -= s[1];
    s[1] += d[1] * 0.5;
    s[0] = 0.0;
    for ( i = 1; i++ < n1; ) {
        s[2] = s0.get();
        d[2] = s0.get();
        d[2] -= s[2];
        s[2] += d[2] * 0.5;
        d[1] += (s[2] + s[0]) * 0.25;
        s1.put(s[1]);
        d1.put(d[1]);
        s[0] = s[1];
        s[1] = s[2];
        d[1] = d[2];
        }
    s[2] = 0.0;
    d[1] += (s[2] + s[0]) * 0.25;
    s1.put(s[1]);
    d1.put(d[1]);
}

void unliftDisk ( int n0, FileOfDouble &s0, FileOfDouble &s1, FileOfDouble &d1 )
    // inverse TS-transform
{
    int n1 = n0 >> 1;
    int i;
        // predict* + S-transform* + merge:
    s0.seek(0);
    s1.seek(0);
    d1.seek(0);
    double s[3], d[3];
    s[0] = 0.0;
    s[1] = s1.get();
    d[1] = d1.get();
    for ( i = 1; i++ < n1; ) {
        s[2] = s1.get();
        d[2] = d1.get();
        d[1] -= (s[2] + s[0]) * 0.25;
        double s1act = s[1] - d[1] * 0.5;
        d[1] += s1act;
        s0.put(s1act);
        s0.put(d[1]);
        s[0] = s[1];
        s[1] = s[2];
        d[1] = d[2];
        }
    s[2] = 0.0;
    d[1] -= (s[2] + s[0]) * 0.25;
    s[1] -= d[1] * 0.5;
    d[1] += s[1];
    s0.put(s[1]);
    s0.put(d[1]);
}

int waveletDisk ( int size )
  // size == 0 .. normal size, == 1 .. big
{
    remove(S0FileName);
    remove(S1FileName);
    remove(D1FileName);
    FileOfDouble s0(S0FileName);
    FileOfDouble s1(S1FileName);
    FileOfDouble d1(D1FileName);
    int maxSize = (size == 0) ? WaveDMaxSize : WaveDMaxSizeBig;
    int total   = maxSize;
    int minSize = (size == 0) ? WaveDMinSize : WaveDMinSizeBig;
    int iter = WaveDIter;
    int i;
    RandomJames rnd;
    startTimer();
    do {
        s0.rewrite();
        s1.rewrite();
        d1.rewrite();
        i = total >> 1;
        do
            s0.put(rnd.uniformNumber());
        while ( --i );
        int elOld = elapsedTime();
        for ( i = 0; i++ < iter; ) {
            liftDisk(total>>1,s0,s1,d1);
            unliftDisk(total>>1,s0,s1,d1);
            }
        int el = elapsedTime();
        printf("%7.2f",(el-elOld)*0.001);
        fflush(stdout);
        iter += iter;
        } while ( (total /= 2) >= minSize );
    int elapsed = elapsedTime();
    s0.close();
    s1.close();
    d1.close();
    remove(S0FileName);
    remove(S1FileName);
    remove(D1FileName);
    if ( superBrief )
        putchar('\r');
    else
        printf("\nWavelet T-S transform on disk: arr: %dMB-%dMB, iter: %d-%d, time: %.2f sec\n",
               (maxSize*sizeof(double))>>20,(minSize*sizeof(double))>>20,WaveDIter,iter>>1,elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  Adaptive K-D tree on disk:

#define TreeFile            "akd-tree.bin"

int akdTree ( int size )
  // size == 0 .. normal size, == 1 .. big
{
    remove(TreeFile);
    startTimer();
    RandomJames rnd;
    AKDtree tree(TreeFile,TreeXhalf,TreeCache);
    int i;
    int treeInsertions = (size == 0) ? TreeInsertions : TreeInsertionsBig;
    int treeOperations = (size == 0) ? TreeOperations : TreeOperationsBig;
    unsigned stat[6];
    for ( i = 0; i++ < treeInsertions; ) {
        int32 x = (int32)(rnd.uniformNumber() * TreeXmax);
        int32 y = (int32)(rnd.uniformNumber() * TreeYmax);
        tree.insert(x,y);
        if ( !(i & 0x1FFF) ) {
            tree.getStat(stat);
            printf("Op: %dK, rd: %uK, wr: %uK, fl: %uK, leaf: %u, hits: %u\r",
                   i/1000,stat[0]/1000,stat[1]/1000,stat[2]/1000,stat[4],stat[5]);
            fflush(stdout);
            }
        }
    double x = TreeXmax * 0.5;
    double y = TreeYmax * 0.5;
    while ( i++ < treeOperations ) {
        double dx = rnd.uniformNumber() * TreeWalkAmp - (TreeWalkAmp/2);
        double dy = rnd.uniformNumber() * TreeWalkAmp - (TreeWalkAmp/2);
        if ( (x += dx) < 1.0 ) x += TreeXmax;
        if ( x >= TreeXmax-1.0 ) x -= TreeXmax;
        if ( (y += dy) < 1.0 ) y += TreeYmax;
        if ( y >= TreeYmax-1.0 ) y -= TreeYmax;
        if ( rnd.uniformNumber() < 1.0/3.0 ) tree.insert((int)x,(int)y);
        else                                 tree.search((int)x,(int)y);
        if ( !(i & 0x1FFF) ) {
            tree.getStat(stat);
            printf("Op: %dK, rd: %uK, wr: %uK, fl: %uK, leaf: %u, hits: %u\r",
                   i/1000,stat[0]/1000,stat[1]/1000,stat[2]/1000,stat[4],stat[5]);
            fflush(stdout);
            }
        }
    tree.getStat(stat);
    tree.close();
    remove(TreeFile);
    int elapsed = elapsedTime();
    if ( !superBrief ) {
        printf("Adaptive K-D tree on disk: operations: %dK, elapsed time: %.2f sec\n",
               treeOperations/1000,elapsed*0.001);
        printf("Block reads: %uK, writes: %uK, flushes: %uK, leaf nodes: %u, hits: %u\n",
               stat[0]/1000,stat[1]/1000,stat[2]/1000,stat[4],stat[5]);
        }
    return elapsed;
}

//-----------------------------------------------------------
//  quick-sort in virtual memory:

bool quickSort ( double *a, int length )
{
    RandomJames rnd;
    int i;
    for ( i = 0; i < length; )              // data init
        a[i++] = rnd.uniformNumber();
    qs(a,length);                           // quick-sort
    for ( i = 1; i++ < length; a++ )        // check
        if ( a[0] > a[1] ) return false;
    return true;
}

int quickSortTest ( int length )
{
    startTimer();
    bool ok;
    int elapsed;
    int repeat = 0;
    do {
        if ( repeat++ && !superBrief ) {
            printf("Pass %d\r",repeat-1);
            fflush(stdout);
            }
        double *a = (double*)VIRTUAL_ALLOC(length*sizeof(double));
        if ( !a ) return 0;
        ok = quickSort(a,length);
        VIRTUAL_FREE(a,length*sizeof(double));
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Quick-sort in virtual memory: %s, double[%dM], elapsed time: %.2f sec\n",
               ok?"OK":"failed",(length>>20),elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  memory management test:

class TreeNode {

public:

    TreeNode *left, *right;

    TreeNode *up;

    byte *data;

    int32 sum;

    TreeNode ( int32 dataSize, TreeNode *parent );

    void regenerate ();

    ~TreeNode ();

protected:

    static int32 arraySize ( int32 length )
    {
        if ( length <= 0 ) return 0;
        return( (length + 12 + 7) & -8 );
    }

    int32 mySize ()
    {
        return( 32 + (data ? arraySize(*((int32*)data)) : 0) );
    }

    };

TreeNode::TreeNode ( int32 dataSize, TreeNode *parent )
{
    left = right = NULL;
    up = parent;
    data = NULL;
    if ( dataSize > 0 ) {
        data =  new byte[dataSize];
        *((int32*)data) = dataSize;
        }
    sum = mySize();
}

void TreeNode::regenerate ()
{
    TreeNode *ptr = this;
    do {
        ptr->sum = ptr->mySize();
        if ( ptr->left )  ptr->sum += ptr->left->sum;
        if ( ptr->right ) ptr->sum += ptr->right->sum;
        } while ( (ptr = ptr->up) );
}

TreeNode::~TreeNode ()
{
    if ( data ) delete []data;
    if ( left ) delete left;
    if ( right ) delete right;
}

int gcTest ()
{
    startTimer();
    int elapsed;
    int repeat = 0;
    unsigned totalAlloc;
    int32 maxAlloc;
    do {
        repeat++;

            // initialization:
        RandomJames rnd;
        TreeNode *root = new TreeNode(0,NULL);
        totalAlloc = maxAlloc = root->sum;
        int i;
        bool grow = false;

        for ( i = 0; i++ < GcIter; ) {

                // path variables:
            int path = (int)(rnd.uniformNumber() * GcMaxPath);
            TreeNode *ptr = root;
            bool change = false;

                // find the hot-spot:
            while ( path-- ) {              // one step forward
                if ( rnd.uniformNumber() < 0.5 ) { // to the left
                    if ( !ptr->left ) {
                        ptr->left = new TreeNode( (int32)(rnd.uniformNumber()*GcVarData + GcMinData), ptr );
                        change = true;
                        }
                    ptr = ptr->left;
                    }
                else {                      // to the right
                    if ( !ptr->right ) {
                        ptr->right = new TreeNode( (int32)(rnd.uniformNumber()*GcVarData + GcMinData), ptr );
                        change = true;
                        }
                    ptr = ptr->right;
                    }
                }

                // truncate the sub-tree:
            if ( rnd.uniformNumber() < 0.5 ) { // truncate the left sub-tree
                if ( ptr->left ) {
                    delete ptr->left;
                    ptr->left = NULL;
                    change = true;
                    }
                }
            else {                          // truncate the right sub-tree
                if ( ptr->right ) {
                    delete ptr->right;
                    ptr->right = NULL;
                    change = true;
                    }
                }

                // regenerate all counters:
            if ( change ) {
                int32 before = root->sum;
                ptr->regenerate();
                int32 after = root->sum;
                if ( after > before ) {     // memory grows
                    totalAlloc += after - before;
                    grow = true;
                    }
                else {                      // memory shortens
                    if ( grow &&            // it has grown before
                         before > maxAlloc ) maxAlloc = before;
                    grow = false;
                    }
                }

            if ( !(i & 0x7FFF) ) LiveCycle(i,GcIter);
            }

        int32 before = root->sum;
        if ( before > maxAlloc ) maxAlloc = before;
        delete root;

        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );

    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Garbage collection test in 16MB of VM (%u MB, max %d KB), time: %.2f sec\n",
               (totalAlloc+(1<<19))>>20,(maxAlloc+512)>>10,elapsed*0.001);
    return elapsed;
}

//-----------------------------------------------------------
//  regular expression test:

int regularTest ()
{
    char buf[RegLen+1];
    buf[RegLen] = (char)0;
    int sum1, sum2, sum3, sum4;
    int elapsed;
    int repeat = 0;
    startTimer();
    do {
        repeat++;
        {
        RandomJames rnd;
        VRegularExpression reg1("(.*a){16,16}.*");              // at least 16 occurances of "a"
        VRegularExpression reg2("(.*aa){8,8}.*");               // at least 8 occurances of "aa"
        VRegularExpression reg3("^(.*a)a[^a]a[^a]a^(a.*)");     // three isolated "a" characters in one sequence
        VRegularExpression reg4(".*(aaaa.*bbbb|bbbb.*aaaa).*"); // both "aaaa" and "bbbb"
        int i, j;
        sum1 = sum2 = sum3 = sum4 = 0;
        for ( i = 0; i++ < RegIter; ) {
            for ( j = 0; j < RegLen; j++ )
                buf[j] = (rnd.uniformNumber() < 0.5) ? 'a' : 'b';
            if ( reg1.match(buf) ) sum1++;
            if ( reg2.match(buf) ) sum2++;
            if ( reg3.match(buf) ) sum3++;
            if ( reg4.match(buf) ) sum4++;
            if ( !(i & 0x7F) ) LiveCycle(i,RegIter);
            }
        }
        elapsed = elapsedTime();
        } while ( elapsed < MinElapsed );
    elapsed = (elapsed + (repeat>>1)) / repeat;
    if ( !superBrief )
        printf("Regular expressions (%dk,%d): %6.4f-%6.4f-%6.4f-%6.4f, time: %.2f sec\n",
               (RegIter<<2)/1000,RegLen,
               (sum1+0.0)/RegIter,(sum2+0.0)/RegIter,(sum3+0.0)/RegIter,(sum4+0.0)/RegIter,elapsed*0.001);
    return elapsed;
}
