/*
    test.h: VUCAKO Bench core (test routines).

    $Rev: 93 $
    $Date: 2007-03-17 16:25:56 +0100 (so, 17 III 2007) $

    Copyright (C) 2001-2007 by Josef Pelikan, MFF UK Prague
        http://cgg.ms.mff.cuni.cz/~pepca/

    VUCAKO Bench is Open Source software (see http://www.opensource.org/
    for description what Open Source is); you may distributed it and/or
    modify it under the terms of GNU General Public License as published
    by the Free Software Foundation; either version 2, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    A copy of the GNU General Public License is available as
    `/usr/doc/copyright/GPL' in the Debian GNU/Linux distribution or on
    the World Wide Web at http://www.gnu.org/copyleft/gpl.html.  You can
    also obtain it by writing to the Free Software Foundation, Inc., 59
    Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _TEST_H
#define _TEST_H

//-----------------------------------------------------------
//  Support:

#define LiveCycle(n,total)  {printf("%3.0f%%\r",((n)*100.0)/(total));fflush(stdout);}

void qs ( double *a, int len );

//-----------------------------------------------------------
//   1. Eratosthen:

#define PrimeOrder        5555555
#define PrimeAssert     100000000

int eratosthen ( int primeOrder, int primeAssert );

//-----------------------------------------------------------
//   2.-3., 17.-18. image transposition:

#define ImageWidth1          4096
#define ImageHeight1         4096
#define ImageIter1           48

#define ImageWidth2          8192
#define ImageHeight2         8192
#define ImageIter2           12

int imageTransposition ( int width, int height, int iter );
int imageTranspositionCorrect ( int width, int height, int iter );

//-----------------------------------------------------------
//   4. arbitrage sequence lookup:

#define MaxN        16
#define ArbitrageN  12
#define MinBound    1.0

void initInstance ();
void solveInstance ();

//-----------------------------------------------------------
//   5. needle throwing simulation:

#define NeedleMax   100000000

int needleSimulation ();

//-----------------------------------------------------------
//   6. fast mem-copy:

#define MemCopyMax   (16<<20)
#define MemCopyIter  100

int memCopy ();

//-----------------------------------------------------------
//   7. Monte-Carlo form-factor between two identical rectangles:

#define FFIter  50000000

int formFactor ();

//-----------------------------------------------------------
//   8., 19. natural merge-sort of huge disk files:

#define SortSize        (1<<23)
#define BucketSize      1024

#define SortSizeBig     (1<<26)
#define BucketSizeBig   8192

int diskSort ( int size );

//-----------------------------------------------------------
//   9. wavelet transform (memory cache test):

#define WaveMaxSize     (1<<22)
#define WaveMinSize     (1<<11)
#ifdef ALT
# define WaveIter        100
#else
# define WaveIter        20
#endif

int waveletMemory ();

//-----------------------------------------------------------
//  10., 20. wavelet transform (disk cache test):

#define WaveDMaxSize    (1<<24)
#define WaveDMinSize    (1<<17)
#define WaveDIter       1

#define WaveDMaxSizeBig (1<<27)
#define WaveDMinSizeBig (1<<20)

int waveletDisk ( int size );

//-----------------------------------------------------------
//  11., 21. adaptive K-D tree on disk:

#define TreeInsertions      50000
#define TreeWalk           100000
#define TreeOperations     (TreeInsertions+TreeWalk)
#define TreeCache            2048
#define TreeXmax            10240
#define TreeYmax            10240
#define TreeWalkAmp          1024
#define TreeXhalf           (TreeXmax>>1)

#define TreeInsertionsBig  500000
#define TreeWalkBig       1000000
#define TreeOperationsBig  (TreeInsertionsBig+TreeWalkBig)

int akdTree ( int size );

//-----------------------------------------------------------
//  12-14. quick-sort in virtual memory:

#define QuickSortLen1    (8<<20)
#define QuickSortLen2   (32<<20)
#define QuickSortLen3  (128<<20)

int quickSortTest ( int length );

//-----------------------------------------------------------
//  15. memory management test:

#define GcIter          1000000
#define GcMaxPath       72
#define GcMinData       12
#define GcVarData       144

int gcTest ();

//-----------------------------------------------------------
//  16. regular expression test:

#define RegIter     4000
#define RegLen      32

int regularTest ();

//-----------------------------------------------------------
//  22.-23. parallel SHA1 digests (1 to 16 threads, 2 threads):

#define ShaDataSize    (1 << 20)
#define ShaDataFiles   4000
#define ShaQueueSize   64
#define ShaMaxThreads  16

#define ShaThreads23   2
#define ShaShift23     13

int parallelSha ( int maxThreads );
int parallelSha23 ();

//-----------------------------------------------------------
//  24. parallel sort in virtual memory:

// Array size: 2x ParallelSortLen
#ifdef ALT
# define ParallelSortLen  (32<<20)
#else
# define ParallelSortLen  QuickSortLen3
#endif
#define PsCellSize       (1<<15)
#define PsMaxThreads     16

int parallelSort ( int maxThreads );

//-----------------------------------------------------------

#endif
